Unit TNT; { c:\langs\delphi32\source\vcl d:\tnt\dev }
{$H+,X+,J+}
Interface
Uses TNTConst,Windows,Messages,SysUtils,Classes,Graphics,Controls,Forms,Dialogs,ExtCtrls,StdCtrls,Buttons;
Type
	{ Base controls }
	TNTControl =
		Class(TCustomControl)
			Published
				Property Align;
				Property ParentShowHint;
				Property ShowHint;
				Property Visible;
		End;

	TNTBaseBevelBorder = (BBBNotPushed,BBBPushed,BBBNone);
	TNTBaseBevel =
		Class(TNTControl)
			Protected
				FBorder					:TNTBaseBevelBorder;
				Procedure SetBorder(_:TNTBaseBevelBorder);
				Procedure Paint; Override;
			Public
				Constructor Create(Owner:TComponent); Override;
		End;

	TNTBaseButton =
		Class(TNTControl)
			Private
				FDown					:Boolean;
				FDragging					:Boolean;
				Procedure SetDown(Value:Boolean);
				Procedure WMLButtonDblClk(Var Message:TWMLButtonDown); Message WM_LBUTTONDBLCLK;
				Procedure CMEnabledChanged(Var Message:TMessage); Message CM_ENABLEDCHANGED;
				Procedure CMButtonPressed(Var Message:TMessage); Message CM_BUTTONPRESSED;
				Procedure CMDialogChar(Var Message:TCMDialogChar); Message CM_DIALOGCHAR;
				Procedure CMFontChanged(Var Message:TMessage); Message CM_FONTCHANGED;
				Procedure CMTextChanged(Var Message:TMessage); Message CM_TEXTCHANGED;
			Protected
				FState					:TButtonState;
				FDefault,FCancel			:Boolean;
				Procedure Loaded; Override;
				Procedure SetDefault(_:Boolean);
				Procedure SetCancel(_:Boolean);
				Procedure MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:Integer); Override;
				Procedure MouseMove(Shift:TShiftState; X,Y:Integer); Override;
				Procedure MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:Integer); Override;
			Public
				Constructor Create(AOwner:TComponent); Override;
				Property Down				:Boolean read FDown write SetDown default False;
				Property Default			:Boolean read FDefault write SetDefault;
				Property Cancel			:Boolean read FCancel write SetCancel;
			Published
				Property Caption;
				Property Enabled;
				Property OnClick;
		End;

	TOnChangedEvent = Procedure(Sender:TObject) of Object;
	TNTBaseScrollBar =
		Class(TNTControl)
			Private
				FMin,FMax,FVal				:Cardinal;
				FActive,FDown				:Boolean;
				FOnChanged				:TOnChangedEvent;
				Procedure SetMax(_:Cardinal);
				Procedure SetMin(_:Cardinal);
				Procedure SetVal(_:Cardinal);
				Procedure MouseDown(O:TObject; Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
				Procedure MouseUp(O:TObject; Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
				Procedure KeyDown(Var Key:Word; Shift:TShiftState); Override;
				Procedure WndProc(Var Message:TMessage); Override;
				Procedure MouseMove(Sender:TObject; Shift:TShiftState; X,Y:Integer); Virtual;
			Public
				Constructor Create(AOwner:TComponent); Override;
				Procedure DoChanged;
			Published
				Property Max				:Cardinal read FMax write SetMax stored True;
				Property Min				:Cardinal read FMin write SetMin stored True;
				Property Val				:Cardinal read FVal write SetVal stored True;
				Property TabOrder;
				Property TabStop			default True;
				Property ShowHint			default True;
				Property Hint;
				Property OnChanged			:TOnChangedEvent read FOnChanged write FOnChanged stored True;
		End;

	{ Published controls }
	TNTBevel =
		Class(TNTBaseBevel)
			Published
				Property Border			:TNTBaseBevelBorder read FBorder write SetBorder;
				Property OnClick;
				Property OnDblClick;
				Property OnMouseDown;
				Property OnMouseMove;
				Property OnMouseUp;
		End;

	TNTButton =
		Class(TNTBaseButton)
			Private
				FAlignment				:TAlignment;
				FActive					:Boolean;
				Procedure SetAlignment(Value:TAlignment);
				Procedure WndProc(Var Message:TMessage); Override;
			Protected
				Procedure Paint; Override;
				Procedure KeyDown(Var Key:Word; Shift:TShiftState); Override;
				Procedure KeyUp(Var Key:Word; Shift:TShiftState); Override;
				Procedure KeyPress(Var Key:Char); Override;
			Public
				Constructor Create(AOwner:TComponent); Override;
			Published
				{Property Default;
				Property Cancel;}
				Property Alignment			:TAlignment read FAlignment write SetAlignment default taLeftJustify;
				Property TabOrder;
				Property TabStop			default True;
		End;

	TNTLabel =
		Class(TCustomLabel)
			Private
				FCaption					:String;
				FBorder					:TNTBaseBevelBorder;
				FAlignment				:TAlignment;
				Procedure SetBorder(_:TNTBaseBevelBorder);
				Procedure SetAlignment(Value:TAlignment);
				Procedure SetCaption(S:String);
				Procedure Paint; Override;
			Public
				Constructor Create(AOwner:TComponent); Override;
			Published
				Property Align;
				Property Alignment			:TAlignment read FAlignment write SetAlignment default taLeftJustify;
				Property Caption			:String read FCaption write SetCaption;
				Property Border			:TNTBaseBevelBorder read FBorder write SetBorder;
				Property ParentShowHint;
				Property ShowHint;
				Property Visible;
				Property OnClick;
		End;

	TNTPanel =
		Class(TCustomPanel)
			Protected
				FBorder					:TNTBaseBevelBorder;
				Procedure SetBorder(_:TNTBaseBevelBorder);
				Procedure Paint; Override;
			Published
				Property Border			:TNTBaseBevelBorder read FBorder write SetBorder;
				Property Alignment;
				Property Enabled;
				Property ParentShowHint;
				Property PopupMenu;
				Property ShowHint;
				Property Hint;
				Property Visible;
				Property OnClick;
				Property OnDblClick;
				Property OnMouseDown;
				Property OnMouseMove;
				Property OnMouseUp;
				Property OnResize;
		End;

	TNTVScrollBar =
		Class(TNTBaseScrollBar)
			Private
				Procedure Paint; Override;
				Procedure MouseMove(Sender:TObject; Shift:TShiftState; X,Y:Integer); Override;
		End;

	TNTHScrollBar =
		Class(TNTBaseScrollBar)
			Private
				Procedure Paint; Override;
				Procedure MouseMove(Sender:TObject; Shift:TShiftState; X,Y:Integer); Override;
		End;

	TNTSmallVScrollBar =
		Class(TNTBaseScrollBar)
			Private
				Procedure Paint; Override;
				Procedure MouseMove(Sender:TObject; Shift:TShiftState; X,Y:Integer); Override;
		End;

	TNTSmallHScrollBar =
		Class(TNTBaseScrollBar)
			Private
				Procedure Paint; Override;
				Procedure MouseMove(Sender:TObject; Shift:TShiftState; X,Y:Integer); Override;
		End;

	TNTMemoLabel =
		Class(TNTControl)
			Private
				FLines					:TStringList;
				FBorder					:TNTBaseBevelBorder;
				FAlignment				:TAlignment;
				Procedure SetBorder(_:TNTBaseBevelBorder);
				Procedure SetAlignment(Value:TAlignment);
				Procedure SetLines(S:TStringList);
				Procedure Paint; Override;
				Procedure Edited(Sender:TObject);
			Public
				Constructor Create(AOwner:TComponent); Override;
				Destructor Destroy; Override;
			Published
				Property Align;
				Property Alignment			:TAlignment read FAlignment write SetAlignment default taLeftJustify;
				Property Lines				:TStringList read FLines write SetLines;
				Property Border			:TNTBaseBevelBorder read FBorder write SetBorder;
				Property ParentShowHint;
				Property ShowHint;
				Property Visible;
				Property OnClick;
		End;
Procedure Register;
Procedure DoRect(C:TCanvas; X,Y,X2,Y2:Integer; P:TNTBaseBevelBorder);
Implementation
Const Alignments:Array[TAlignment] of Word = (DT_Left,DT_Right,DT_Center);
Procedure DoRect(C:TCanvas; X,Y,X2,Y2:Integer; P:TNTBaseBevelBorder);
	Begin
		With C do Begin
			{Brush.Color := TNTBkg;}
			Brush.Style := bsSolid;
			Pen.Style := psSolid;
			Pen.Width := 1;
			FillRect(Rect(X,Y,X2 - X,Y2 - Y));
			If P = BBBNone then
				Exit;
			Dec(X2);
			Dec(Y2);
			If P = BBBNotPushed then
				Pen.Color := TNTBCol2
			Else
				Pen.Color := TNTBCol1;
			MoveTo(X,Y2);
			LineTo(X,Y);
			LineTo(X2,Y);
			If P = BBBNotPushed then
				Pen.Color := TNTBCol1
			Else
				Pen.Color := TNTBCol2;
			LineTo(X2,Y2);
			LineTo(X,Y2);
		End;
	End;
Constructor TNTBaseBevel.Create(Owner:TComponent);
	Begin
		Inherited Create(Owner);
		FBorder := BBBPushed;
		ControlStyle := [csOpaque];
	End;
Procedure TNTBaseBevel.Paint;
	Begin
		Canvas.Brush.Color := TNTBkg;
		DoRect(Canvas,0,0,Width,Height,FBorder);
	End;
Procedure TNTBaseBevel.SetBorder(_:TNTBaseBevelBorder);
	Begin
		FBorder := _;
		Paint;
	End;
Constructor TNTBaseButton.Create(AOwner:TComponent);
	Begin
		Inherited Create(AOwner);
		SetBounds(0,0,25,25);
		ControlStyle := [csCaptureMouse,csOpaque,csDoubleClicks];
		ParentFont := True;
	End;
Procedure TNTBaseButton.Loaded;
	Var State:TButtonState;
	Begin
		Inherited Loaded;
		If Enabled then
			State := bsUp
		Else
			State := bsDisabled;
	End;
Procedure TNTBaseButton.MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
	Begin
		Inherited MouseDown(Button,Shift,X,Y);
		SetFocus;
		If (Button = mbLeft) and Enabled then Begin
			If not FDown then Begin
				FState := bsDown;
				Paint;
			End;
			FDragging := True;
		End;
	End;
Procedure TNTBaseButton.MouseMove(Shift:TShiftState; X,Y:Integer);
	Var NewState:TButtonState;
	Begin
		Inherited MouseMove(Shift,X,Y);
		If FDragging then Begin
			If not FDown then
				NewState := bsUp
			Else
				NewState := bsExclusive;
			If (X >= 0) and (X < ClientWidth) and (Y >= 0) and (Y <= ClientHeight) then
				If FDown then
				NewState := bsExclusive
				Else
					NewState := bsDown;
			If NewState <> FState then Begin
				FState := NewState;
				Paint;
			End;
		End;
	End;
Procedure TNTBaseButton.MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
	Var DoClick:Boolean;
	Begin
		Inherited MouseUp(Button,Shift,X,Y);
		If FDragging then Begin
			FDragging := False;
			DoClick := (X >= 0) and (X < ClientWidth) and (Y >= 0) and (Y <= ClientHeight);
			FState := bsUp;
			If DoClick then
				SetDown(not FDown)
			Else Begin
				If FDown then
					FState := bsExclusive;
				Paint;
			End;
			FDown := False;
			If DoClick then
				Click;
		End;
	End;
Procedure TNTBaseButton.SetDown(Value:Boolean);
	Begin
		If Value <> FDown then Begin
			FDown := Value;
			If Value then
				FState := bsExclusive
			Else
				FState := bsUp;
			Invalidate;
		End;
	End;
Procedure TNTBaseButton.WMLButtonDblClk(Var Message:TWMLButtonDown);
	Begin
		Inherited;
		if FDown then
			DblClick;
	End;
procedure TNTBaseButton.CMEnabledChanged(Var Message:TMessage);
	Begin
		Invalidate;
	End;
procedure TNTBaseButton.CMButtonPressed(Var Message:TMessage);
	Var Sender:TNTBaseButton;
	Begin
		Sender := TNTBaseButton(Message.LParam);
		If Sender <> Self then Begin
			If Sender.Down and FDown then Begin
				FDown := False;
				FState := bsUp;
				Invalidate;
			End;
		End;
	End;
Procedure TNTBaseButton.CMDialogChar(Var Message:TCMDialogChar);
	Begin
		With Message do
			If IsAccel(CharCode,Caption) and Enabled and Visible then Begin
				Click;
				Result := 1;
			End Else
				Inherited;
	End;
Procedure TNTBaseButton.CMFontChanged(Var Message:TMessage);
	Begin
		Invalidate;
	End;
Procedure TNTBaseButton.CMTextChanged(Var Message:TMessage);
	Begin
		Invalidate;
	End;
Procedure TNTBaseButton.SetDefault(_:Boolean);
	Begin
		FDefault := _;
		Paint;
	End;
Procedure TNTBaseButton.SetCancel(_:Boolean);
	Begin
		FCancel := _;
		Paint;
	End;
Constructor TNTButton.Create(AOwner:TComponent);
	Begin
		Inherited Create(AOwner);
	End;
Procedure TNTButton.Paint;
	Var
		S		:String;
		R		:TRect;
		C		:TColor;
	Procedure DoBorder(Color:TColor);
		Begin
			With Canvas do Begin
				Brush.Style := bsClear;
				Pen.Color := Color;
				Pen.Style := psSolid;
				Pen.Mode := pmCopy;
				Pen.Width := 1;
				Rectangle(R.Left,R.Top,R.Right,R.Bottom);
				Inc(R.Left);
				Inc(R.Top);
				Dec(R.Right);
				Dec(R.Bottom);
			End;
		End;
	Procedure DecIt;
		Begin
			Inc(R.Left,2);
			Inc(R.Top,2);
			Dec(R.Right,2);
			Dec(R.Bottom,2);
		End;
	Begin
		With Canvas do Begin
			S := Caption;
			R := ClientRect;
			C := TNTBkg;
			If FDefault then Begin
				S := 'ENTER:' + S;
				C := TNTSBorder;
			End;
			If FCancel then Begin
				S := 'ESC:' + S;
				C := TNTSBorder;
			End;
			If FActive then
				DoBorder(TNTABorder);
			Brush.Color := C;
			If FState = bsDown then Begin
				DoRect(Canvas,R.Left,R.Top,R.Right,R.Bottom,BBBPushed);
				DecIt;
				Font.Color := TNTDownTextC;
				Inc(R.Left,1);
				Inc(R.Top,1);
				DrawText(Handle,PChar(S),Length(S),R,(DT_ExpandTabs or DT_WordBreak) or Alignments[FAlignment]);
			End Else Begin
				DoRect(Canvas,R.Left,R.Top,R.Right,R.Bottom,BBBNotPushed);
				DecIt;
				Font.Color := TNTTextColor;
				DrawText(Handle,PChar(S),Length(S),R,(DT_ExpandTabs or DT_WordBreak) or Alignments[FAlignment]);
			End;
		End;
	End;
Procedure TNTButton.SetAlignment(Value:TAlignment);
	Begin
		If FAlignment <> Value then Begin
			FAlignment := Value;
			Paint;
		End;
	End;
Procedure TNTButton.WndProc(Var Message:TMessage);
	Begin
		Case Message.Msg of
			WM_SetFocus:
				Begin
					FActive := True;
					Paint;
				End;
			WM_KillFocus:
				Begin
					FActive := False;
					FState := bsUp;
					Paint;
				End;
			Else
				Inherited WndProc(Message);
		End;
	End;
Procedure TNTButton.KeyDown(Var Key:Word; Shift:TShiftState);
	Begin
		If (FState = bsUp) and ((Key = 13) or (Key = 32)) then Begin
			FState := bsDown;
			Paint;
		End;
	End;
Procedure TNTButton.KeyUp(Var Key:Word; Shift:TShiftState);
	Begin
		If (FState = bsDown) and ((Key = 13) or (Key = 32)) then Begin
			FState := bsUp;
			Paint;
			Click;
		End;
	End;
Procedure TNTButton.KeyPress(Var Key:Char);
	Begin
		If ((Key = #13) and (FDefault)) or ((Key = #27) and (FCancel)) then Begin
			SetFocus;
			Click;
		End;
	End;
Constructor TNTLabel.Create(AOwner:TComponent);
	Begin
		Inherited Create(AOwner);
		AutoSize := False;
		FBorder := BBBNotPushed;
		ControlStyle := [csOpaque,csCaptureMouse];
	End;
Procedure TNTLabel.Paint;
	Var
		Rect		:TRect;
		S		:String;
	Begin
		Canvas.Brush.Color := TNTBkg;
		DoRect(Canvas,0,0,Width,Height,FBorder);
		With Canvas do Begin
			Rect := ClientRect;
			Rect.Left := Rect.Left + 2;
			Rect.Top := Rect.Top + 2;
			Rect.Right := Rect.Right - 2;
			Rect.Bottom := Rect.Bottom - 2;
			If FBorder = BBBPushed then
				Font.Color := TNTDownTextC
			Else
				Font.Color := TNTTextColor;
			DrawText(Handle,PChar(FCaption),Length(FCaption),Rect,(DT_ExpandTabs or DT_WordBreak) or Alignments[FAlignment]);
		End;
	End;
Procedure TNTLabel.SetBorder(_:TNTBaseBevelBorder);
	Begin
		FBorder := _;
		Paint;
	End;
Procedure TNTLabel.SetAlignment(Value:TAlignment);
	Begin
		If FAlignment <> Value then Begin
			FAlignment := Value;
			Paint;
		End;
	End;
Procedure TNTLabel.SetCaption(S:String);
	Begin
		FCaption := S;
		Paint;
	End;
Procedure TNTPanel.SetBorder(_:TNTBaseBevelBorder);
	Begin
		FBorder := _;
		Refresh;
	End;
Procedure TNTPanel.Paint;
	Begin
		Canvas.Brush.Color := TNTBkg;
		DoRect(Canvas,0,0,Width,Height,FBorder);
	End;
Constructor TNTBaseScrollBar.Create(AOwner:TComponent);
	Begin
		Inherited Create(AOwner);
		Hint := 'PGUP/PGDOWN';
		OnMouseDown := MouseDown;
		OnMouseMove := MouseMove;
		OnMouseUp := MouseUp;
	End;
Procedure TNTBaseScrollBar.SetMax(_:Cardinal);
	Begin
		FMax := _;
		Paint;
	End;
Procedure TNTBaseScrollBar.SetMin(_:Cardinal);
	Begin
		FMin := _;
		Paint;
	End;
Procedure TNTBaseScrollBar.SetVal(_:Cardinal);
	Begin
		FVal := _;
		Paint;
	End;
Procedure TNTBaseScrollBar.WndProc(Var Message:TMessage);
	Begin
		Case Message.Msg of
			WM_SetFocus:
				Begin
					FActive := True;
					Paint;
				End;
			WM_KillFocus:
				Begin
					FActive := False;
					Paint;
				End;
			Else
				Inherited WndProc(Message);
		End;
	End;
Procedure TNTBaseScrollBar.MouseDown(O:TObject; Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
	Begin
		FDown := True;
		SetFocus;
		MouseMove(O,Shift,X,Y);
	End;
Procedure TNTBaseScrollBar.MouseUp(O:TObject; Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
	Begin
		FDown := False;
		Paint;
	End;
Procedure TNTBaseScrollBar.KeyDown(Var Key:Word; Shift:TShiftState);
	Procedure Trig;
		Begin
			If Assigned(FOnChanged) then
				FOnChanged(Self);
		End;
	Begin
		Case Key of
			33:
				Begin
					Val := Val - 1;
					Paint;
					Trig;
				End;
			34:
				Begin
					Val := Val + 1;
					Paint;
					Trig;
				End;
			35:
				Begin
					Val := FMax;
					Trig;
				End;
			36:
				Begin
					Val := FMin;
					Trig;
				End;
			45:
				Begin
					Val := FVal - Round((FMax - FMin) * (Height / 10));
					Trig;
				End;
			46:
				Begin
				End;
			Else
				Inherited;
		End;
	End;
Procedure TNTBaseScrollBar.MouseMove(Sender:TObject; Shift:TShiftState; X,Y:Integer);
	Begin
	End;
Procedure TNTBaseScrollBar.DoChanged;
	Begin
		If Assigned(FOnChanged) then
			FOnChanged(Self);
	End;
Procedure TNTVScrollBar.Paint;
	Var
		V	:Single;
		T	:Integer;
	Begin
		If FMax < FMin then
			FMax := FMin;
		If FVal < FMin then
			FVal := FMin;
		If FVal > FMax then
			FVal := FMax;
		Canvas.Brush.Color := TNTBkg;
		DoRect(Canvas,0,0,Width,Height,BBBPushed);
		If FActive then
			With Canvas do Begin
				Pen.Color := TNTABorder;
				Rectangle(0,0,Width,Height);
			End;
		If FMin = FMax then
			V := 0
		Else
			V := (FVal - FMin) / (FMax - FMin);
		T := Round((Height - 18) * V) + 1;
		If not FDown then
			DoRect(Canvas,1,T,Width - 1,T + 16,BBBNotPushed)
		Else
			DoRect(Canvas,1,T,Width - 1,T + 16,BBBPushed);
	End;
Procedure TNTVScrollBar.MouseMove(Sender:TObject; Shift:TShiftState; X,Y:Integer);
	Var
		V	:Single;
		N	:Cardinal;
	Begin
		If not FDown then
			Exit;
		Y := Y - 8;
		If Y < 0 then
			Y := 0;
		If Y > Height - 16 then
			Y := Height - 16;
		V := Y / (Height - 16);
		N := Round((V * (FMax - FMin)) + FMin);
		If N <> FVal then Begin
			FVal := N;
			DoChanged;
			Paint;
		End;
	End;
Procedure TNTHScrollBar.Paint;
	Var
		V	:Single;
		T	:Integer;
	Begin
		If FMax < FMin then
			FMax := FMin;
		If FVal < FMin then
			FVal := FMin;
		If FVal > FMax then
			FVal := FMax;
		Canvas.Brush.Color := TNTBkg;
		DoRect(Canvas,0,0,Width,Height,BBBPushed);
		If FActive then
			With Canvas do Begin
				Pen.Color := TNTABorder;
				Rectangle(0,0,Width,Height);
			End;
		If FMin = FMax then
			V := 0
		Else
			V := (FVal - FMin) / (FMax - FMin);
		T := Round((Width - 18) * V) + 1;
		If not FDown then
			DoRect(Canvas,T,1,T + 16,Height - 1,BBBNotPushed)
		Else
			DoRect(Canvas,T,1,T + 16,Height - 1,BBBPushed);
	End;
Procedure TNTHScrollBar.MouseMove(Sender:TObject; Shift:TShiftState; X,Y:Integer);
	Var
		V	:Single;
		N	:Cardinal;
	Begin
		If not FDown then
			Exit;
		X := X - 8;
		If X < 0 then
			X := 0;
		If X > Width - 16 then
			X := Width - 16;
		V := X / (Width - 16);
		N := Round((V * (FMax - FMin)) + FMin);
		If N <> FVal then Begin
			FVal := N;
			DoChanged;
			Paint;
		End;
	End;
Procedure TNTSmallVScrollBar.Paint;
	Var
		V	:Single;
		T	:Integer;
	Begin
		If FMax < FMin then
			FMax := FMin;
		If FVal < FMin then
			FVal := FMin;
		If FVal > FMax then
			FVal := FMax;
		Canvas.Brush.Color := TNTBkg;
		DoRect(Canvas,0,0,Width,Height,BBBPushed);
		If FActive then
			With Canvas do Begin
				Pen.Color := TNTABorder;
				Rectangle(0,0,Width,Height);
			End;
		If FMin = FMax then
			V := 0
		Else
			V := (FVal - FMin) / (FMax - FMin);
		T := Round((Height - 10) * V) + 1;
		If not FDown then
			DoRect(Canvas,1,T,Width - 1,T + 8,BBBNotPushed)
		Else
			DoRect(Canvas,1,T,Width - 1,T + 8,BBBPushed);
	End;
Procedure TNTSmallVScrollBar.MouseMove(Sender:TObject; Shift:TShiftState; X,Y:Integer);
	Var
		V	:Single;
		N	:Cardinal;
	Begin
		If not FDown then
			Exit;
		Y := Y - 4;
		If Y < 0 then
			Y := 0;
		If Y > Height - 8 then
			Y := Height - 8;
		V := Y / (Height - 8);
		N := Round((V * (FMax - FMin)) + FMin);
		If N <> FVal then Begin
			FVal := N;
			DoChanged;
			Paint;
		End;
	End;
Procedure TNTSmallHScrollBar.Paint;
	Var
		V	:Single;
		T	:Integer;
	Begin
		If FMax < FMin then
			FMax := FMin;
		If FVal < FMin then
			FVal := FMin;
		If FVal > FMax then
			FVal := FMax;
		Canvas.Brush.Color := TNTBkg;
		DoRect(Canvas,0,0,Width,Height,BBBPushed);
		If FActive then
			With Canvas do Begin
				Pen.Color := TNTABorder;
				Rectangle(0,0,Width,Height);
			End;
		If FMin = FMax then
			V := 0
		Else
			V := (FVal - FMin) / (FMax - FMin);
		T := Round((Width - 10) * V) + 1;
		If not FDown then
			DoRect(Canvas,T,1,T + 8,Height - 1,BBBNotPushed)
		Else
			DoRect(Canvas,T,1,T + 8,Height - 1,BBBPushed);
	End;
Procedure TNTSmallHScrollBar.MouseMove(Sender:TObject; Shift:TShiftState; X,Y:Integer);
	Var
		V	:Single;
		N	:Cardinal;
	Begin
		If not FDown then
			Exit;
		X := X - 8;
		If X < 0 then
			X := 0;
		If X > Width - 16 then
			X := Width - 16;
		V := X / (Width - 16);
		N := Round((V * (FMax - FMin)) + FMin);
		If N <> FVal then Begin
			FVal := N;
			DoChanged;
			Paint;
		End;
	End;
Constructor TNTMemoLabel.Create(AOwner:TComponent);
	Begin
		Inherited Create(AOwner);
		FBorder := BBBNotPushed;
		ControlStyle := [csOpaque,csCaptureMouse];
		FLines := TStringList.Create;
		FLines.OnChange := Edited;
	End;
Procedure TNTMemoLabel.Paint;
	Var
		Rect		:TRect;
		S		:String;
	Begin
		Canvas.Brush.Color := TNTBkg;
		DoRect(Canvas,0,0,Width,Height,FBorder);
		With Canvas do Begin
			Rect := ClientRect;
			Rect.Left := Rect.Left + 2;
			Rect.Top := Rect.Top + 2;
			Rect.Right := Rect.Right - 2;
			Rect.Bottom := Rect.Bottom - 2;
			If FBorder = BBBPushed then
				Font.Color := TNTDownTextC
			Else
				Font.Color := TNTTextColor;
			S := FLines.GetText;
			DrawText(Handle,PChar(S),Length(S),Rect,(DT_ExpandTabs or DT_WordBreak) or Alignments[FAlignment]);
		End;
	End;
Procedure TNTMemoLabel.SetBorder(_:TNTBaseBevelBorder);
	Begin
		FBorder := _;
		Paint;
	End;
Procedure TNTMemoLabel.SetAlignment(Value:TAlignment);
	Begin
		If FAlignment <> Value then Begin
			FAlignment := Value;
			Paint;
		End;
	End;
Procedure TNTMemoLabel.SetLines(S:TStringList);
	Begin
		FLines.Assign(S);
		Refresh;
	End;
Procedure TNTMemoLabel.Edited(Sender:TObject);
	Var
		Rect		:TRect;
		S		:String;
	Begin
		Canvas.Brush.Color := TNTBkg;
		DoRect(Canvas,1,1,Width - 1,Height - 1,BBBNone);
		With Canvas do Begin
			Rect := ClientRect;
			Rect.Left := Rect.Left + 2;
			Rect.Top := Rect.Top + 2;
			Rect.Right := Rect.Right - 2;
			Rect.Bottom := Rect.Bottom - 2;
			If FBorder = BBBPushed then
				Font.Color := TNTDownTextC
			Else
				Font.Color := TNTTextColor;
			S := FLines.GetText;
			DrawText(Handle,PChar(S),Length(S),Rect,(DT_ExpandTabs or DT_WordBreak) or Alignments[FAlignment]);
		End;
	End;
Destructor TNTMemoLabel.Destroy;
	Begin
		FLines.Free;
		Inherited Destroy;
	End;
Procedure Register;
	Begin
		RegisterComponents('TNT', [TNTBevel,TNTButton,TNTLabel,TNTPanel,TNTVScrollBar,TNTHScrollBar,TNTSmallVScrollBar,TNTSmallHScrollBar,TNTMemoLabel]);
	End;
Initialization
	{MessageBox(Application.Handle,'TNT 0.02 Alpha - customized GUI-controls for Delphi 2.0.'#13#10'Do not redistrubute!'#13#10#13#10'Contact: pcmicke@2.sbbs.se (pcmicke@rocketmail.com & pcmicke@hotmail.com optionals)','TNT control pack',32);}
End.
